require(nls2)
require(dplyr)
require(stringr)
require(ggplot2)
require(Hmisc)
require(beepr)
require(lubridate)
require(MCMCglmm)
require(devtools)
devtools::install_github('ontogenerator/cpdetectorr')
require(cpdetectorr)

# folder <- choose.dir(caption = "Select folder with ChoiceOutput and other csv files to load")
folder <- "C:\\!Vladi\\Manuscripts\\Repeatability" #or directly set the working directory to the right path
setwd(folder)

is.odd <- function(x) x %% 2 != 0 # function returning TRUE for odd numbers, FALSE for even

# function for plotting autocorrelations from https://github.com/tmalsburg/MCMCglmm-intro
plot.acfs <- function(x) {
  n <- dim(x)[2]
  par(mfrow=c(ceiling(n/2),2), mar=c(3,2,3,0))
  for (i in 1:n) {
    acf(x[,i], lag.max=100, main=colnames(x)[i])
    grid()
  }
}

#read main_data file
main_data <- read.csv2(file="main_data.csv", header=TRUE, dec=".",sep=";", 
                          na.strings="NA") %>% # convert data time format
  mutate(DateTime = as.POSIXct(DateTime, origin="1899-12-30", tz="GMT"))

# read metadata for bat individuals
bat_data <- read.csv2(file="animals.csv", header=TRUE, dec=".",sep=";", 
                       na.strings="NA")

main_data <- main_data %>% group_by(day, Animal) %>% # calculate time intervals between
  # consecutive visits, as well as each visit's number
  mutate(t_diff = as.double(DateTime - lag(DateTime)), n_vis = 1:n()) 

########## Figure 1
# density plot of all time intervals for all bats on a logarithmic axis
main_data %>% filter(!is.na(t_diff), day < 46) %>%
  ggplot(aes(t_diff, group = Animal)) + geom_density(geom = "line", fill = "black", color = "transparent", alpha = 0.05) +
  scale_x_log10(name = "time interval between two consecutive visits [s]") +
  geom_vline(xintercept = 120, linetype = 2) + theme_bw(base_size = 12, base_family = "") +
  annotate("text", label = "within bout", x = 10, y = 1.5, size = 6) +
  annotate("text", label = "between bout", x = 3000, y = 1.5, size = 6)

# set bout break criterion to 120 seconds
b_crit <- 120
# classify visits as within or between-bouts and number bouts
main_data <- main_data %>% arrange(day, Animal, DateTime) %>%
  mutate(new_bout = if_else(is.na(t_diff), 1, if_else(t_diff < b_crit, 0, 1)),
         bout = cumsum(new_bout))


# filter out data after 00:00h from nights on which nectar was prematurely depleted
main_data <- main_data %>% filter(filter_t == 0 | hour(DateTime)>16)

# for the main days of analysis, calculate the number of flowers visited, number
# of visits, number of bouts and visits per bout, then remove nights on which a bat
# made less than 24 visits (only these bats' data are removed)
summary_tab <- main_data %>% group_by(day, intensity, Animal) %>% filter(day < 46) %>%
  summarise(n_flowers = n_distinct(loc), n_tot = n(),
            n_bouts = max(bout), vis_bout = n_tot/n_bouts) %>%
  filter(n_tot > 23)

# display the bats that only have data from two nights
summary_tab %>% group_by(Animal) %>% summarise(nights = n()) %>%
  filter(nights < 3)

# remove these bats from the summary table
summary_tab <- summary_tab %>%
  filter(!(Animal %in% c("Bat10", "Bat43", "Bat59", "Bat64"))) %>% inner_join(bat_data) %>%
  mutate(rep_st = factor(rep_st))

summary_tab %>% filter(rep_st == 1) %>% group_by(Animal) %>% summarise(n_preg = n())

# Check which animals produced sufficient data on sufficient number of nights
unique(summary_tab$Animal)

# calculate the mean durations for these bats only for the rewarded visits they made
durs <- main_data %>% filter(Animal %in% unique(summary_tab$Animal), vol == 60) %>% group_by(day, Animal) %>%
   summarise(dur = mean(Sense1duration))

#add durations to original dataset
summary_tab <- summary_tab %>% inner_join(durs)

# plot visits per bout for female and male bats
summary_tab %>% ggplot(aes(Animal, vis_bout, color = sex)) + geom_boxplot() +
  scale_x_discrete() + theme(axis.text.x=element_text(angle=90,hjust=1))
# plot visit duration for female and male bats
summary_tab %>% ggplot(aes(Animal, dur, color = sex)) + geom_boxplot() +
  scale_x_discrete() + theme(axis.text.x=element_text(angle=90,hjust=1))
# plot number of flowers visited for female and male bats
summary_tab %>% ggplot(aes(Animal, n_flowers, color = sex)) + geom_boxplot() +
  scale_x_discrete() + theme(axis.text.x=element_text(angle=90,hjust=1))

# calculate the number of female and male individuals that generated sufficient data
summary_tab %>% group_by(Animal, sex) %>% summarise(n = n()) %>%
  group_by(sex) %>% summarise(n = n())

# summaries for number of repeated measures
summary_tab %>% group_by(Animal, sex) %>% summarise(n_nights = n()) %>% ungroup() %>%
  summarise(mean = mean(n_nights), sd = sd(n_nights), min = min(n_nights), max = max(n_nights))

# range of average number of flowers visited
summary_tab %>% group_by(Animal, sex) %>% summarise(n_flowers = mean(n_flowers)) %>%
  group_by(sex) %>% summarise(min_nf = min(n_flowers), max_nf = max(n_flowers))


######### Table 1. phenotypic correlations and p values
corrs <- summary_tab %>% group_by(Animal, sex) %>% summarise(vis_bout = mean(vis_bout),
                                                    dur = mean(dur), n_flowers = mean(n_flowers))
corrs %>% ungroup() %>% 
  filter(sex == "male") %>% select(-Animal, -sex) %>% as.matrix() %>%
  rcorr(type = "pearson")

corrs %>% ungroup() %>% 
  filter(sex == "female") %>% select(-Animal, -sex) %>% as.matrix() %>%
  rcorr(type = "pearson")

# summary statistics about the three main behavioral measures
corrs %>% group_by(sex) %>% summarise_if(is.numeric, funs(mean, sd))

set.seed(5872)
prior.univar <-list(R=list(V=1, nu=0.002), G=list(G1=list(V=1, nu=0.002)))

vb.mcmc<-MCMCglmm(vis_bout~ intensity+sex+fa_length, random=~us(intensity):Animal,
                 family="gaussian", prior=prior.univar,
                 nitt=130000,thin=100, burnin=30000,
                 data=as.data.frame(summary_tab),verbose=FALSE)
summary(vb.mcmc)
geweke.plot(vb.mcmc$VCV)
autocorr(vb.mcmc$VCV) %>% as_tibble() %>% slice(2:n()) %>%
  filter_all(any_vars(.>0.1)) %>% summarise(n = n())#should have absolute lag <0.10 for Lag >0  #should have absolute lag <0.10 for Lag >0
plot(vb.mcmc$Sol, auto.layout=F)
plot.acfs(vb.mcmc$Sol)

set.seed(2863)
d.mcmc<-MCMCglmm(dur~ intensity+sex+fa_length, random=~us(intensity):Animal,
                 family="gaussian", prior=prior.univar,
                 nitt=130000,thin=100, burnin=30000,
                 data=as.data.frame(summary_tab),verbose=FALSE)
summary(d.mcmc)
geweke.plot(d.mcmc$VCV)
autocorr(d.mcmc$VCV) %>% as_tibble() %>% slice(2:n()) %>%
  filter_all(any_vars(.>0.1)) %>% summarise(n = n()) #should have absolute lag <0.10 for Lag >0
plot(d.mcmc$Sol, auto.layout=F)
plot.acfs(d.mcmc$Sol)

set.seed(1729)
nf.mcmc<-MCMCglmm(n_flowers~ intensity+sex+fa_length, random=~us(intensity):Animal,
                 family="poisson", prior=prior.univar,
                 nitt=130000,thin=100, burnin=30000,
                 data=as.data.frame(summary_tab),verbose=FALSE)
summary(nf.mcmc)
geweke.plot(nf.mcmc$VCV)
autocorr(nf.mcmc$VCV) %>% as_tibble() %>% slice(2:n()) %>%
  filter_all(any_vars(.>0.1)) %>% summarise(n = n())#should have absolute lag <0.10 for Lag >0  #should have absolute lag <0.10 for Lag >0
plot(nf.mcmc$Sol, auto.layout=F)
plot.acfs(nf.mcmc$Sol)


########## check to see if pregnant females differ from non-pregnant ones
set.seed(1465)
pr.d.mcmc<-MCMCglmm(dur~ intensity+fa_length+rep_st, random=~us(intensity):Animal,
                 family="gaussian", prior=prior.univar,
                 nitt=130000,thin=100, burnin=30000,
                 data=as.data.frame(summary_tab %>% filter(sex == "female")),verbose=FALSE)
summary(pr.d.mcmc)
geweke.plot(pr.d.mcmc$VCV)
autocorr(pr.d.mcmc$VCV) %>% as_tibble() %>% slice(2:n()) %>%
  filter_all(any_vars(.>0.1)) %>% summarise(n = n()) #should have absolute lag <0.10 for Lag >0
plot(pr.d.mcmc$Sol, auto.layout=F)
plot.acfs(pr.d.mcmc$Sol)


set.seed(9265)
pr.vb.mcmc<-MCMCglmm(vis_bout~ intensity+fa_length+rep_st, random=~us(intensity):Animal,
                  family="gaussian", prior=prior.univar,
                  nitt=130000,thin=100, burnin=30000,
                  data=as.data.frame(summary_tab %>% filter(sex == "female")),verbose=FALSE)
summary(pr.vb.mcmc)
geweke.plot(pr.vb.mcmc$VCV)
autocorr(pr.vb.mcmc$VCV) %>% as_tibble() %>% slice(2:n()) %>%
  filter_all(any_vars(.>0.1)) %>% summarise(n = n())#should have absolute lag <0.10 for Lag >0  #should have absolute lag <0.10 for Lag >0
plot(pr.vb.mcmc$Sol, auto.layout=F)
plot.acfs(pr.vb.mcmc$Sol)

set.seed(7634)
pr.nf.mcmc<-MCMCglmm(n_flowers~ intensity+fa_length+rep_st, random=~us(intensity):Animal,
                  family="poisson", prior=prior.univar,
                  nitt=130000,thin=100, burnin=30000,
                  data=as.data.frame(summary_tab %>% filter(sex == "female")),verbose=FALSE)
summary(pr.nf.mcmc)
geweke.plot(pr.nf.mcmc$VCV)
autocorr(pr.nf.mcmc$VCV) %>% as_tibble() %>% slice(2:n()) %>%
  filter_all(any_vars(.>0.1)) %>% summarise(n = n())#should have absolute lag <0.10 for Lag >0  #should have absolute lag <0.10 for Lag >0
plot(pr.nf.mcmc$Sol, auto.layout=F)
plot.acfs(pr.nf.mcmc$Sol)


# visualize the effect of intensity on all three behavioral measures
summary_tab %>% ggplot(aes(intensity, vis_bout, color = Animal, linetype = sex)) +
  stat_smooth(method = lm, se = F)
summary_tab %>% ggplot(aes(intensity, dur, color = Animal, linetype = sex)) +
  stat_smooth(method = lm, se = F)
summary_tab %>% ggplot(aes(intensity, n_flowers, color = Animal, linetype = sex)) +
  stat_smooth(method = lm, se = F)

####### Priors for trivariate model (a good model should be robust to the choice of prior)
#inverse Wishart
# prior.trivar<-list(R=list(V=diag(3),nu=2.002),G=list(G1=list(V=diag(3), nu=2.002)))
#parameter expanded
prior.trivar<-list(R=list(V=diag(3),nu=2.002),
G=list(G1=list(V=diag(3), nu=3, alpha.mu=rep(0,3), alpha.V=diag(3)*1000)))
# flat prior
# prior.trivar<-list(R=list(V=diag(3)*0.002,nu=0.002),
#                    G=list(G1=list(V=diag(3)*0.002,nu=0.002)))

###############MALES
set.seed(4285)
# warning!!! this tests takes a long time to run
males.trivarC.mcmc<-
  MCMCglmm(cbind(vis_bout,dur,n_flowers)~(trait-1) + intensity, 
           random=~us(trait):Animal,rcov=~us(trait):units,
           family=c("gaussian","gaussian", "poisson"), prior=prior.trivar,
           nitt=130000,thin=100,burnin=30000,
           data=as.data.frame(summary_tab %>% filter(sex == "male")),verbose=FALSE)


summary(males.trivarC.mcmc)
beep(6) # model takes a long time to calculate. beep announces when ready
geweke.plot(males.trivarC.mcmc$VCV) # few points should fall outside of interval if ok convergence
autocorr(males.trivarC.mcmc$VCV) %>% as_tibble() %>% slice(2:n()) %>%
  filter_all(any_vars(.>0.1)) %>% summarise(n = n())#should have absolute lag <0.10 for Lag >0
plot(males.trivarC..mcmc$Sol, auto.layout=F)
plot.acfs(males.trivarC..mcmc$Sol)

set.seed(5437)
#### fixed between-correlations to 0
# warning!!! this tests takes a long time to run
nocorr.males.trivarC.mcmc<-
  MCMCglmm(cbind(vis_bout,dur,n_flowers)~(trait-1) + intensity, 
           random=~idh(trait):Animal,rcov=~us(trait):units,
           family=c("gaussian","gaussian", "poisson"), prior=prior.trivar,
           nitt=130000,thin=100,burnin=30000,
           data=as.data.frame(summary_tab %>% filter(sex == "male")),verbose=FALSE)


summary(nocorr.males.trivarC.mcmc)
beep(6)
geweke.plot(nocorr.males.trivarC.mcmc$VCV)
autocorr(nocorr.males.trivarC.mcmc$VCV) %>% as_tibble() %>% slice(2:n()) %>%
  filter_all(any_vars(.>0.1)) %>% summarise(n = n())

#Delta DIC for no correlation between dep. variables and correlations freely estimated
summary(nocorr.males.trivarC.mcmc)$DIC - summary(males.trivarC.mcmc)$DIC


rep.d <- males.trivarC.mcmc$VCV[,"traitdur:traitdur.Animal"]/
  (males.trivarC.mcmc$VCV[,"traitdur:traitdur.Animal"] +
     males.trivarC.mcmc$VCV[,"traitdur:traitdur.units"])
rep.nf <- males.trivarC.mcmc$VCV[,"traitn_flowers:traitn_flowers.Animal"]/
  (males.trivarC.mcmc$VCV[,"traitn_flowers:traitn_flowers.Animal"] +
     males.trivarC.mcmc$VCV[,"traitn_flowers:traitn_flowers.units"])
rep.vb <- males.trivarC.mcmc$VCV[,"traitvis_bout:traitvis_bout.Animal"]/
  (males.trivarC.mcmc$VCV[,"traitvis_bout:traitvis_bout.Animal"] +
     males.trivarC.mcmc$VCV[,"traitvis_bout:traitvis_bout.units"])

cor.vb.d <- males.trivarC.mcmc$VCV[,"traitdur:traitvis_bout.Animal"]/
  sqrt(males.trivarC.mcmc$VCV[,"traitvis_bout:traitvis_bout.Animal"]*
         males.trivarC.mcmc$VCV[,"traitdur:traitdur.Animal"])
cor.vb.nf <- males.trivarC.mcmc$VCV[,"traitn_flowers:traitvis_bout.Animal"]/
  sqrt(males.trivarC.mcmc$VCV[,"traitvis_bout:traitvis_bout.Animal"]*
         males.trivarC.mcmc$VCV[,"traitn_flowers:traitn_flowers.Animal"])
cor.d.nf <- males.trivarC.mcmc$VCV[,"traitdur:traitn_flowers.Animal"]/
  sqrt(males.trivarC.mcmc$VCV[,"traitdur:traitdur.Animal"]*
         males.trivarC.mcmc$VCV[,"traitn_flowers:traitn_flowers.Animal"])

cor.vb.d.units <- males.trivarC.mcmc$VCV[,"traitdur:traitvis_bout.units"]/
  sqrt(males.trivarC.mcmc$VCV[,"traitvis_bout:traitvis_bout.units"]*
         males.trivarC.mcmc$VCV[,"traitdur:traitdur.units"])
cor.vb.nf.units <-males.trivarC.mcmc$VCV[,"traitn_flowers:traitvis_bout.units"]/
  sqrt(males.trivarC.mcmc$VCV[,"traitvis_bout:traitvis_bout.units"]*
         males.trivarC.mcmc$VCV[,"traitn_flowers:traitn_flowers.units"])
cor.d.nf.units <-males.trivarC.mcmc$VCV[,"traitdur:traitn_flowers.units"]/
  sqrt(males.trivarC.mcmc$VCV[,"traitdur:traitdur.units"]*
         males.trivarC.mcmc$VCV[,"traitn_flowers:traitn_flowers.units"])

#posterior mode for repeatability with 95% credibility intervals
males.reps <- tibble(vb = c(posterior.mode(rep.vb), HPDinterval(rep.vb)),
                   d = c(posterior.mode(rep.d), HPDinterval(rep.d)),
                   nf = c(posterior.mode(rep.nf), HPDinterval(rep.nf))) %>%
  mutate_all(funs(round(.,2)))
# correlations for males
males.corr.table <- tibble(vb_d_inter = c(posterior.mode(cor.vb.d), HPDinterval(cor.vb.d)),
                     vb_nf_inter = c(posterior.mode(cor.vb.nf), HPDinterval(cor.vb.nf)),
                     d_nf_inter = c(posterior.mode(cor.d.nf), HPDinterval(cor.d.nf)),
                     vb_d_intra = c(posterior.mode(cor.vb.d.units), HPDinterval(cor.vb.d.units)),
                     vb_nf_intra = c(posterior.mode(cor.vb.nf.units), HPDinterval(cor.vb.nf.units)),
                     d_nf_intra = c(posterior.mode(cor.d.nf.units), HPDinterval(cor.d.nf.units))) %>%
  mutate_all(funs(round(.,2)))


##############FEMALES
set.seed(2973)
females.trivarC.mcmc<-
  MCMCglmm(cbind(vis_bout,dur,n_flowers)~(trait-1) + intensity, 
           random=~us(trait):Animal,rcov=~us(trait):units,
           family=c("gaussian","gaussian", "poisson"), prior=prior.trivar,
           nitt=130000,thin=100,burnin=30000,
           data=as.data.frame(summary_tab %>% filter(sex == "female")),verbose=FALSE)

beep(6)
summary(females.trivarC.mcmc)
geweke.plot(females.trivarC.mcmc$VCV)
autocorr(females.trivarC.mcmc$VCV) %>% as_tibble() %>% slice(2:n()) %>%
  filter_all(any_vars(.>0.1)) %>% summarise(n = n())
plot(females.trivarC..mcmc$Sol, auto.layout=F)
plot.acfs(females.trivarC..mcmc$Sol)

rep.d <- females.trivarC.mcmc$VCV[,"traitdur:traitdur.Animal"]/
  (females.trivarC.mcmc$VCV[,"traitdur:traitdur.Animal"] +
     females.trivarC.mcmc$VCV[,"traitdur:traitdur.units"])
rep.nf <- females.trivarC.mcmc$VCV[,"traitn_flowers:traitn_flowers.Animal"]/
  (females.trivarC.mcmc$VCV[,"traitn_flowers:traitn_flowers.Animal"] +
     females.trivarC.mcmc$VCV[,"traitn_flowers:traitn_flowers.units"])
rep.vb <- females.trivarC.mcmc$VCV[,"traitvis_bout:traitvis_bout.Animal"]/
  (females.trivarC.mcmc$VCV[,"traitvis_bout:traitvis_bout.Animal"] +
     females.trivarC.mcmc$VCV[,"traitvis_bout:traitvis_bout.units"])

cor.vb.d <- females.trivarC.mcmc$VCV[,"traitdur:traitvis_bout.Animal"]/
  sqrt(females.trivarC.mcmc$VCV[,"traitvis_bout:traitvis_bout.Animal"]*
         females.trivarC.mcmc$VCV[,"traitdur:traitdur.Animal"])
cor.vb.nf <- females.trivarC.mcmc$VCV[,"traitn_flowers:traitvis_bout.Animal"]/
  sqrt(females.trivarC.mcmc$VCV[,"traitvis_bout:traitvis_bout.Animal"]*
         females.trivarC.mcmc$VCV[,"traitn_flowers:traitn_flowers.Animal"])
cor.d.nf <- females.trivarC.mcmc$VCV[,"traitdur:traitn_flowers.Animal"]/
  sqrt(females.trivarC.mcmc$VCV[,"traitdur:traitdur.Animal"]*
         females.trivarC.mcmc$VCV[,"traitn_flowers:traitn_flowers.Animal"])

cor.vb.d.units <- females.trivarC.mcmc$VCV[,"traitdur:traitvis_bout.units"]/
  sqrt(females.trivarC.mcmc$VCV[,"traitvis_bout:traitvis_bout.units"]*
         females.trivarC.mcmc$VCV[,"traitdur:traitdur.units"])
cor.vb.nf.units <-females.trivarC.mcmc$VCV[,"traitn_flowers:traitvis_bout.units"]/
  sqrt(females.trivarC.mcmc$VCV[,"traitvis_bout:traitvis_bout.units"]*
         females.trivarC.mcmc$VCV[,"traitn_flowers:traitn_flowers.units"])
cor.d.nf.units <-females.trivarC.mcmc$VCV[,"traitdur:traitn_flowers.units"]/
  sqrt(females.trivarC.mcmc$VCV[,"traitdur:traitdur.units"]*
         females.trivarC.mcmc$VCV[,"traitn_flowers:traitn_flowers.units"])

#posterior mode for repeatability with 95% credibility intervals
females.reps <- tibble(vb = c(posterior.mode(rep.vb), HPDinterval(rep.vb)),
                     d = c(posterior.mode(rep.d), HPDinterval(rep.d)),
                     nf = c(posterior.mode(rep.nf), HPDinterval(rep.nf))) %>%
  mutate_all(funs(round(.,2)))

# correlations for females
females.corr.table <- tibble(vb_d_inter = c(posterior.mode(cor.vb.d), HPDinterval(cor.vb.d)),
                           vb_nf_inter = c(posterior.mode(cor.vb.nf), HPDinterval(cor.vb.nf)),
                           d_nf_inter = c(posterior.mode(cor.d.nf), HPDinterval(cor.d.nf)),
                           vb_d_intra = c(posterior.mode(cor.vb.d.units), HPDinterval(cor.vb.d.units)),
                           vb_nf_intra = c(posterior.mode(cor.vb.nf.units), HPDinterval(cor.vb.nf.units)),
                           d_nf_intra = c(posterior.mode(cor.d.nf.units), HPDinterval(cor.d.nf.units))) %>%
  mutate_all(funs(round(.,2)))

# Table 2. Display correlation tables
males.corr.table
females.corr.table

# Repeatability estimates
males.reps
females.reps

set.seed(9653)
#### fixed between-correlations to 0
nocorr.females.trivarC.mcmc<-
  MCMCglmm(cbind(vis_bout,dur,n_flowers)~(trait-1) + intensity, 
           random=~idh(trait):Animal,rcov=~us(trait):units,
           family=c("gaussian","gaussian", "poisson"), prior=prior.trivar,
           nitt=130000,thin=100,burnin=30000,
           data=as.data.frame(summary_tab %>% filter(sex == "female")),verbose=FALSE)


summary(nocorr.females.trivarC.mcmc)
beep(6)
geweke.plot(nocorr.females.trivarC.mcmc$VCV)
autocorr(nocorr.females.trivarC.mcmc$VCV) %>% as_tibble() %>% slice(2:n()) %>%
  filter_all(any_vars(.>0.1)) %>% summarise(n = n())

#Delta DIC for no correlation between dep. variables and correlations freely estimated
summary(nocorr.females.trivarC.mcmc)$DIC - summary(females.trivarC.mcmc)$DIC


#visualize significant correlations
summary_tab %>% ggplot(aes(dur, n_flowers, color = Animal, linetype = sex)) + stat_smooth(method = lm, se = F)
summary_tab %>% ggplot(aes(vis_bout, dur, color = Animal, linetype = sex)) + stat_smooth(method = lm, se = F)
summary_tab %>% ggplot(aes(vis_bout, n_flowers, color = Animal, linetype = sex)) + stat_smooth(method = lm, se = F)

############ Analyses of foraging performance
# a list of the seven day numbers, on which the highest relative intensities were tested
days_hi_int <- c(31,32,33,34,35,36,37)

# extract the subset from the main data that corresponds to these nights, and only consider
# bats that made >23 visits on more than two nights
hi_int <- main_data %>% inner_join(bat_data) %>% filter(day %in% days_hi_int) %>% group_by(day,Animal, sex) %>%
  summarise(tot_vis = n()) %>% filter(tot_vis > 23)
n_lo_int <- hi_int %>% group_by(Animal, sex) %>% summarise(n = n()) %>% filter(n < 3) %>%
  select(Animal) %>% unlist()

hi_qual <- main_data %>% filter(!Animal %in% !!n_lo_int, day %in% !!days_hi_int) %>%
  group_by(day, Animal) %>% mutate(cumprof = cumsum(prof)) %>%
  group_by(day, Animal, bout) %>% mutate(mean_bout = mean(prof))

hi_qual <- hi_qual %>%
  group_by(rounded_hour = floor_date(DateTime, unit = "hour"), Animal, day) %>%
  mutate(hourlyprof = mean(prof)) %>% inner_join(bat_data)

hi_qual %>% ggplot(aes(bout, mean_bout, color = Animal)) +
  geom_line() + facet_grid(day ~ .)

# calculate the bouts to criterion performance measure. It is defined as the number of the bout that
# passed the criterion for the second time in a row, that is, the first bout with discr. perf. >= criterion for which also
# the previous bout had a discr. perf. >= criterion. 

criterion <- 0.8 # if the criterion is too high, not every bat makes it and there is less data, if too low
# all bats reach it easily and there is less variation between bats
hi_qual_bouts <- hi_qual %>% group_by(Animal, sex, day, intensity, bout) %>% summarise(mean_bout = mean(prof)) %>%
  mutate(first_cb = if_else(mean_bout >= criterion, 1, 0),
         second_cb = lag(first_cb) + first_cb) %>% filter(second_cb == 2) %>% summarise(crit_bout = min(bout)) 

# on average criterion is reached faster at higher intensity 
# (at lower int, males also seem to reach it faster than females)
hi_qual_bouts %>%
  ggplot(aes(intensity, crit_bout, color = sex)) +
  stat_summary(fun.data= "mean_cl_boot", size = 0.6)


# find all change points for all bats and all days (from days_hi_int)
hi_qual_cps <- hi_qual %>% group_by(Animal, sex, day, intensity) %>%
   do(cp_wrapper(.$prof, isDiscrete = TRUE, "ttest", 2))


# find last change point for each bat
errors <- hi_qual_cps %>% group_by(Animal, sex, day, intensity) %>%
  arrange(desc(Trial), .by_group = TRUE) %>% mutate(err_rate = 1 - Slopes,
                                                    CumSs = CumSs-lead(CumSs),
                                                    Trial = Trial - lead(Trial),
                                                    err = Trial - CumSs) %>% slice(1) 

errors %>%
  ggplot(aes(intensity, err_rate, color = sex)) +
  stat_summary(fun.data= "mean_cl_boot", size = 0.6)

# bats that didn't make the criterion:
left_join(errors, hi_qual_bouts) %>% filter(is.na(crit_bout))

# calculate performance measures, 
# removing data for nights on which bats did not reach the criterion
perf <- left_join(errors, hi_qual_bouts) %>% ungroup() %>% filter(!is.na(crit_bout)) %>%
  select(Animal, sex, day, intensity, Trial, CumSs, err, err_rate, crit_bout)

# add performance measure to main table of data
perf.full <- summary_tab %>% left_join(perf)

# check for how many bats there are at least 3 nights of performance measures
perf.full %>% group_by(Animal, sex) %>% filter(!is.na(crit_bout)) %>% summarise(n = n()) %>%
  group_by(sex) %>% summarise(n = n())

# average number of repeated measures of foraging performance
perf.full %>% filter(!is.na(crit_bout)) %>% group_by(Animal) %>%
  summarise(cnt = n()) %>% summarise(n = n(), mean_n = mean(cnt), sd_n = sd(cnt))

# prior definition
prior.penta <-list(R=list(V=diag(5),nu=4.002),G=list(G1=list(V=diag(5), nu=4.002)))
# parameter expanded prior works ok, but not for repeatability estimates
# prior.penta<-list(R=list(V=diag(5),nu=4.002),
#                    G=list(G1=list(V=diag(5), nu=5, alpha.mu=rep(0,5), alpha.V=diag(5)*1000)))
set.seed(4697)
# model for performance measures and correlations with main three measures, for males only
perf.full.mcmc<-MCMCglmm(cbind(crit_bout, cbind(err, CumSs),vis_bout,dur,n_flowers)~
                           (trait-1) + intensity,
                         random=~us(trait):Animal, rcov=~us(trait):units,
                         family=c("poisson", "multinomial2", "gaussian","gaussian", "poisson"),
                         prior=prior.penta, nitt=1300000,thin=1000,burnin=30000,
                         data=as.data.frame(perf.full) %>% filter(sex == "male"), verbose=FALSE)
summary(perf.full.mcmc)
geweke.plot(perf.full.mcmc$VCV)
plot(perf.full.mcmc$Sol, auto.layout=F)
plot.acfs(perf.full.mcmc$Sol)
beep(6)



# repeatability of duration
rep.d <- perf.full.mcmc$VCV[,"traitdur:traitdur.Animal"]/
  (perf.full.mcmc$VCV[,"traitdur:traitdur.Animal"] +
     perf.full.mcmc$VCV[,"traitdur:traitdur.units"])
#repeatability of number of flowers visited
rep.nf <- perf.full.mcmc$VCV[,"traitn_flowers:traitn_flowers.Animal"]/
  (perf.full.mcmc$VCV[,"traitn_flowers:traitn_flowers.Animal"] +
     perf.full.mcmc$VCV[,"traitn_flowers:traitn_flowers.units"])
# repeatability of visits per bout
rep.vb <- perf.full.mcmc$VCV[,"traitvis_bout:traitvis_bout.Animal"]/
  (perf.full.mcmc$VCV[,"traitvis_bout:traitvis_bout.Animal"] +
     perf.full.mcmc$VCV[,"traitvis_bout:traitvis_bout.units"])


# repeatability of trials to criterion
rep.crit_bout <- perf.full.mcmc$VCV[,"traitcrit_bout:traitcrit_bout.Animal"]/(
  perf.full.mcmc$VCV[,"traitcrit_bout:traitcrit_bout.Animal"] +
    perf.full.mcmc$VCV[,"traitcrit_bout:traitcrit_bout.units"]
)

# repeatability of error rate
rep.err <- perf.full.mcmc$VCV[,"traiterr:traiterr.Animal"]/(
  perf.full.mcmc$VCV[,"traiterr:traiterr.Animal"] +
    perf.full.mcmc$VCV[,"traiterr:traiterr.units"]
)

reps <- tibble(vb = c(posterior.mode(rep.vb), HPDinterval(rep.vb)),
               d = c(posterior.mode(rep.d), HPDinterval(rep.d)),
               nf = c(posterior.mode(rep.nf), HPDinterval(rep.nf)),
               err = c(posterior.mode(rep.err), HPDinterval(rep.err)),
               crit_bout = c(posterior.mode(rep.crit_bout), HPDinterval(rep.crit_bout))) %>%
  mutate_all(funs(round(.,2)))

reps

# correlations between individuals
cor.err.ncrit <- perf.full.mcmc$VCV[,"traiterr:traitcrit_bout.Animal"]/
  sqrt(perf.full.mcmc$VCV[,"traitcrit_bout:traitcrit_bout.Animal"]*
         perf.full.mcmc$VCV[,"traiterr:traiterr.Animal"])

cor.vb.ncrit <- perf.full.mcmc$VCV[,"traitvis_bout:traitcrit_bout.Animal"]/
  sqrt(perf.full.mcmc$VCV[,"traitcrit_bout:traitcrit_bout.Animal"]*
         perf.full.mcmc$VCV[,"traitvis_bout:traitvis_bout.Animal"])

cor.nf.ncrit <- perf.full.mcmc$VCV[,"traitn_flowers:traitcrit_bout.Animal"]/
  sqrt(perf.full.mcmc$VCV[,"traitcrit_bout:traitcrit_bout.Animal"]*
         perf.full.mcmc$VCV[,"traitn_flowers:traitn_flowers.Animal"])

cor.d.ncrit <- perf.full.mcmc$VCV[,"traitdur:traitcrit_bout.Animal"]/
  sqrt(perf.full.mcmc$VCV[,"traitcrit_bout:traitcrit_bout.Animal"]*
         perf.full.mcmc$VCV[,"traitdur:traitdur.Animal"])

cor.vb.err <- perf.full.mcmc$VCV[,"traitvis_bout:traiterr.Animal"]/
  sqrt(perf.full.mcmc$VCV[,"traiterr:traiterr.Animal"]*
         perf.full.mcmc$VCV[,"traitvis_bout:traitvis_bout.Animal"])

cor.nf.err <- perf.full.mcmc$VCV[,"traitn_flowers:traiterr.Animal"]/
  sqrt(perf.full.mcmc$VCV[,"traiterr:traiterr.Animal"]*
         perf.full.mcmc$VCV[,"traitn_flowers:traitn_flowers.Animal"])

cor.d.err <- perf.full.mcmc$VCV[,"traitdur:traiterr.Animal"]/
  sqrt(perf.full.mcmc$VCV[,"traiterr:traiterr.Animal"]*
         perf.full.mcmc$VCV[,"traitdur:traitdur.Animal"])

# correlations within individuals
cor.err.ncrit.units <- perf.full.mcmc$VCV[,"traiterr:traitcrit_bout.units"]/
  sqrt(perf.full.mcmc$VCV[,"traitcrit_bout:traitcrit_bout.units"]*
         perf.full.mcmc$VCV[,"traiterr:traiterr.units"])

cor.vb.ncrit.units <- perf.full.mcmc$VCV[,"traitvis_bout:traitcrit_bout.units"]/
  sqrt(perf.full.mcmc$VCV[,"traitcrit_bout:traitcrit_bout.units"]*
         perf.full.mcmc$VCV[,"traitvis_bout:traitvis_bout.units"])

cor.nf.ncrit.units <- perf.full.mcmc$VCV[,"traitn_flowers:traitcrit_bout.units"]/
  sqrt(perf.full.mcmc$VCV[,"traitcrit_bout:traitcrit_bout.units"]*
         perf.full.mcmc$VCV[,"traitn_flowers:traitn_flowers.units"])

cor.d.ncrit.units <- perf.full.mcmc$VCV[,"traitdur:traitcrit_bout.units"]/
  sqrt(perf.full.mcmc$VCV[,"traitcrit_bout:traitcrit_bout.units"]*
         perf.full.mcmc$VCV[,"traitdur:traitdur.units"])


cor.vb.err.units <- perf.full.mcmc$VCV[,"traitvis_bout:traiterr.units"]/
  sqrt(perf.full.mcmc$VCV[,"traiterr:traiterr.units"]*
         perf.full.mcmc$VCV[,"traitvis_bout:traitvis_bout.units"])

cor.nf.err.units <- perf.full.mcmc$VCV[,"traitn_flowers:traiterr.units"]/
  sqrt(perf.full.mcmc$VCV[,"traiterr:traiterr.units"]*
         perf.full.mcmc$VCV[,"traitn_flowers:traitn_flowers.units"])

cor.d.err.units <- perf.full.mcmc$VCV[,"traitdur:traiterr.units"]/
  sqrt(perf.full.mcmc$VCV[,"traiterr:traiterr.units"]*
         perf.full.mcmc$VCV[,"traitdur:traitdur.units"])

cor.vb.d <- perf.full.mcmc$VCV[,"traitdur:traitvis_bout.Animal"]/
  sqrt(perf.full.mcmc$VCV[,"traitvis_bout:traitvis_bout.Animal"]*
         perf.full.mcmc$VCV[,"traitdur:traitdur.Animal"])
cor.vb.nf <- perf.full.mcmc$VCV[,"traitn_flowers:traitvis_bout.Animal"]/
  sqrt(perf.full.mcmc$VCV[,"traitvis_bout:traitvis_bout.Animal"]*
         perf.full.mcmc$VCV[,"traitn_flowers:traitn_flowers.Animal"])
cor.d.nf <- perf.full.mcmc$VCV[,"traitdur:traitn_flowers.Animal"]/
  sqrt(perf.full.mcmc$VCV[,"traitdur:traitdur.Animal"]*
         perf.full.mcmc$VCV[,"traitn_flowers:traitn_flowers.Animal"])

cor.vb.d.units <- perf.full.mcmc$VCV[,"traitdur:traitvis_bout.units"]/
  sqrt(perf.full.mcmc$VCV[,"traitvis_bout:traitvis_bout.units"]*
         perf.full.mcmc$VCV[,"traitdur:traitdur.units"])
cor.vb.nf.units <-perf.full.mcmc$VCV[,"traitn_flowers:traitvis_bout.units"]/
  sqrt(perf.full.mcmc$VCV[,"traitvis_bout:traitvis_bout.units"]*
         perf.full.mcmc$VCV[,"traitn_flowers:traitn_flowers.units"])
cor.d.nf.units <-perf.full.mcmc$VCV[,"traitdur:traitn_flowers.units"]/
  sqrt(perf.full.mcmc$VCV[,"traitdur:traitdur.units"]*
         perf.full.mcmc$VCV[,"traitn_flowers:traitn_flowers.units"])


perf.corrs <- tibble(err_ncrit_inter = c(posterior.mode(cor.err.ncrit), HPDinterval(cor.err.ncrit)),
                     vb_ncrit_inter = c(posterior.mode(cor.vb.ncrit), HPDinterval(cor.vb.ncrit)),
                     nf_ncrit_inter = c(posterior.mode(cor.nf.ncrit), HPDinterval(cor.nf.ncrit)),
                     d_ncrit_inter = c(posterior.mode(cor.d.ncrit), HPDinterval(cor.d.ncrit)),
                     vb_err_inter = c(posterior.mode(cor.vb.err), HPDinterval(cor.vb.err)),
                     nf_err_inter = c(posterior.mode(cor.nf.err), HPDinterval(cor.nf.err)),
                     d_err_inter = c(posterior.mode(cor.d.err), HPDinterval(cor.d.err)),
                     vb_nf_inter = c(posterior.mode(cor.vb.nf), HPDinterval(cor.vb.nf)),
                     vb_d_inter = c(posterior.mode(cor.vb.d), HPDinterval(cor.vb.d)),
                     d_nf_inter = c(posterior.mode(cor.d.nf), HPDinterval(cor.d.nf)),
                     
                     
                     err_ncrit_intra = c(posterior.mode(cor.err.ncrit.units), HPDinterval(cor.err.ncrit.units)),
                     vb_ncrit_intra = c(posterior.mode(cor.vb.ncrit.units), HPDinterval(cor.vb.ncrit.units)),
                     nf_ncrit_intra = c(posterior.mode(cor.nf.ncrit.units), HPDinterval(cor.nf.ncrit)),
                     d_ncrit_intra = c(posterior.mode(cor.d.ncrit.units), HPDinterval(cor.d.ncrit.units)),
                     vb_err_intra = c(posterior.mode(cor.vb.err.units), HPDinterval(cor.vb.err.units)),
                     nf_err_intra = c(posterior.mode(cor.nf.err.units), HPDinterval(cor.nf.err.units)),
                     d_err_intra = c(posterior.mode(cor.d.err.units), HPDinterval(cor.d.err.units)),
                     vb_nf_intra = c(posterior.mode(cor.vb.nf.units), HPDinterval(cor.vb.nf.units)),
                     vb_d_intra = c(posterior.mode(cor.vb.d.units), HPDinterval(cor.vb.d.units)),
                     d_nf_intra = c(posterior.mode(cor.d.nf.units), HPDinterval(cor.d.nf.units))) %>%
  mutate_all(funs(round(.,2)))


# no correlation betwen errror rate and trials to criterion
perf.corrs

# visualisation of the correlations
perf.full %>% ggplot(aes(dur, err, color = Animal, linetype = sex)) + geom_point() +
  geom_smooth(method = "lm", se = FALSE)


# p value correction for multiple testing, using fals discovery rate correction
ps <- c(0.43, 0.008, 0.001, 0.76, 0.94, 0.02, 0.001, 0.04, 0.29)
round(p.adjust(ps, "BH"), 3)
